# -*- coding: utf-8 -*-
"""Subscribers."""

from plone import api
from Products.CMFPlone.utils import base_hasattr

from .browser.settings import get_settings
from . import TYPE_CONFIG


def object_added(obj, event):
    """ Increment the value in registry """
    # internal_number is unknown or empty => no need to increment
    if not base_hasattr(obj, 'internal_number') or not obj.internal_number:
        return
    settings = get_settings()
    pt = obj.portal_type
    if pt in settings:
        settings[pt]['nb'] += 1
    elif 'glo_bal' in settings:
        settings['glo_bal']['nb'] += 1
    else:
        return
    config = []
    for pt in sorted(settings.keys()):
        config.append({'portal_type': pt, 'uniqueness': settings[pt]['u'], 'default_number': settings[pt]['nb'],
                       'default_expression': settings[pt]['expr']})
    api.portal.set_registry_record(TYPE_CONFIG, config)



# -*- coding: utf-8 -*-

from zope.component import adapts
from zope.interface import implements

from plone.indexer import indexer
from Products.CMFCore.interfaces import IContentish
from Products.CMFPlone.utils import base_hasattr, safe_unicode
from Products.PluginIndexes.common.UnIndex import _marker as common_marker

from collective import dexteritytextindexer

from .behavior import IInternalNumberBehavior


@indexer(IContentish)
def internal_number_index(obj):
    """ Index method escaping acquisition and ready for ZCatalog 3 """
    if base_hasattr(obj, 'internal_number') and obj.internal_number:
        return obj.internal_number
    return common_marker


class InternalNumberSearchableExtender(object):
    """
        Extends SearchableText of IInternalNumberBehavior objects.
    """
    adapts(IInternalNumberBehavior)
    implements(dexteritytextindexer.IDynamicTextIndexExtender)

    def __init__(self, context):
        self.context = context

    def __call__(self):
        return safe_unicode(self.context.internal_number)



# -*- coding: utf-8 -*-

from zope import component
from zope import schema
from zope.interface import alsoProvides, Invalid
from z3c.form import validator, widget

from plone import api
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model

from Products.CMFPlone.utils import safe_unicode
from collective.behavior.talcondition.utils import _evaluateExpression

from . import _
from browser.settings import get_pt_settings


class IInternalNumberBehavior(model.Schema):

    internal_number = schema.TextLine(
        title=_(u"Internal number"),
        required=False,)

alsoProvides(IInternalNumberBehavior, IFormFieldProvider)


def validateIndexValueUniqueness(context, portal_type, index_name, value):
    """
        check at 'portal_type' 'context' creation if 'index' 'value' is uniqueness
    """
    # if the value is empty, we don't check anything
    if not value:
        return
    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog.searchResults(**{index_name: value})
    if context.portal_type != portal_type:
        # we create the object, the context is the container
        if brains:
            raise Invalid(_(u"This value '${value}' is already used in ${obj}",
                            mapping={'obj': ', '.join(['<a href="%s" target="_blank">%s</a>' %
                                                       (safe_unicode(b.getURL()), safe_unicode(b.Title))
                                                       for b in brains]),
                                     'value': safe_unicode(value)}))
    else:
        # we edit the object, the context is itself
        # if multiple brains (normally not possible), we are sure there are other objects with same index value
        if len(brains) > 1 or (len(brains) == 1 and brains[0].UID != context.UID()):
            raise Invalid(_(u"This value '${value}' is already used in ${obj}",
                            mapping={'obj': ', '.join(['<a href="%s" target="_blank">%s</a>' %
                                                       (safe_unicode(b.getURL()), safe_unicode(b.Title))
                                                       for b in brains]),
                                     'value': safe_unicode(value)}))


class InternalNumberValidator(validator.SimpleFieldValidator):
    def validate(self, value):
        # we call the already defined validators
        # super(InternalNumberValidator, self).validate(value)
        if not get_pt_settings(self.view.portal_type).get('u', True):
            return
        validateIndexValueUniqueness(self.context, self.view.portal_type, 'internal_number', value)


validator.WidgetValidatorDiscriminators(InternalNumberValidator, field=IInternalNumberBehavior['internal_number'])
component.provideAdapter(InternalNumberValidator)


# To avoid grokking the package required by plone.directives.form.default_value decorator

def internal_number_default(data):
    """ Default value of internal_number """
    settings = get_pt_settings(data.view.portal_type)
    ret = _evaluateExpression(data.context, settings.get('expr', ''), extra_expr_ctx={'number': settings.get('nb', 0)},
                              empty_expr_is_true='')
    return ret

DefaultinternalNumber = widget.ComputedWidgetAttribute(internal_number_default,
                                                       field=IInternalNumberBehavior['internal_number'])

component.provideAdapter(DefaultinternalNumber, name='default')



# -*- coding: utf-8 -*-

from zope import schema
from zope.component import getAllUtilitiesRegisteredFor, getUtility
from zope.i18n.interfaces import ITranslationDomain
from zope.interface import Interface, implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from z3c.form import form

from plone import api
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.autoform.directives import widget
from plone.dexterity.interfaces import IDexterityFTI
from plone.z3cform import layout

from collective.z3cform.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.registry import DictRow

from .. import _, TYPE_CONFIG


class IPortalTypeConfigSchema(Interface):
    portal_type = schema.Choice(
        title=_("Portal type"),
        vocabulary=u'collective.internalnumber.portaltypevocabulary',
        required=True)
    uniqueness = schema.Bool(
        title=_("Uniqueness"),
        required=False)
    default_number = schema.Int(
        title=_(u'Number of next content item'),
        description=_(u"This value can be used as 'number' variable in tal expression"),
        default=1)
    default_expression = schema.TextLine(
        title=_("Default value tal expression"),
        description=_("Elements 'number', 'member', 'context' and 'portal' are available."),
        default=u"number",
        required=False)


class IInternalNumberConfig(Interface):
    """
    Configuration of internalnumber
    """

    portal_type_config = schema.List(
        title=_(u'By type configuration'),
        value_type=DictRow(title=_("Portal type conf"),
                           schema=IPortalTypeConfigSchema))

    widget('portal_type_config', DataGridFieldFactory)


class SettingsEditForm(RegistryEditForm):
    """
    Define form logic
    """
    form.extends(RegistryEditForm)
    schema = IInternalNumberConfig
    label = _("Internal number behavior configuration")

SettingsView = layout.wrap_form(SettingsEditForm, ControlPanelFormWrapper)


def get_settings():
    ptc = api.portal.get_registry_record(TYPE_CONFIG)
    settings = {}
    if ptc is None:
        return settings
    for row in ptc:
        expr = row['default_expression'] and row['default_expression'] or u''
        settings[row['portal_type']] = {'u': row['uniqueness'], 'nb': row['default_number'], 'expr': expr}
    return settings


def get_pt_settings(pt):
    settings = get_settings()
    if pt in settings:
        return settings[pt]
    elif 'glo_bal' in settings:
        return settings['glo_bal']
    return {}


class DxPortalTypesVocabulary(object):
    """ Active mail types vocabulary """
    implements(IVocabularyFactory)

    def __call__(self, context):
        terms = [SimpleTerm('glo_bal', 'glo_bal', _(u'Global configuration'))]
        ftis = getAllUtilitiesRegisteredFor(IDexterityFTI)
        for fti in ftis:
            translation_domain = getUtility(ITranslationDomain, fti.i18n_domain)
            terms.append(SimpleTerm(fti.id, fti.id, translation_domain.translate(fti.title,
                                                                                 context=api.portal.get().REQUEST)))
        return SimpleVocabulary(terms)



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
import unittest

from zope.schema._bootstrapinterfaces import WrongType

from collective.behavior.internalnumber.testing import COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING  # noqa
from plone import api

from .. import TYPE_CONFIG


class TestSetup(unittest.TestCase):
    """Test that collective.behavior.internalnumber is properly installed."""

    layer = COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if collective.behavior.internalnumber is installed."""
        self.assertTrue(self.installer.isProductInstalled(
            'collective.behavior.internalnumber'))

    def test_browserlayer(self):
        """Test that ICollectiveBehaviorInternalnumberLayer is registered."""
        from collective.behavior.internalnumber.interfaces import (
            ICollectiveBehaviorInternalnumberLayer)
        from plone.browserlayer import utils
        self.assertIn(ICollectiveBehaviorInternalnumberLayer, utils.registered_layers())

    def test_registry(self):
        self.assertRaises(api.exc.InvalidParameterError, api.portal.set_registry_record, 'Unexistent key', True)
        self.assertRaises(WrongType, api.portal.set_registry_record, TYPE_CONFIG, 'string')
        api.portal.set_registry_record(TYPE_CONFIG, [])


class TestUninstall(unittest.TestCase):

    layer = COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.uninstallProducts(['collective.behavior.internalnumber'])

    def test_product_uninstalled(self):
        """Test if collective.behavior.internalnumber is cleanly uninstalled."""
        self.assertFalse(self.installer.isProductInstalled(
            'collective.behavior.internalnumber'))

    def test_browserlayer_removed(self):
        """Test that ICollectiveBehaviorInternalnumberLayer is removed."""
        from collective.behavior.internalnumber.interfaces import ICollectiveBehaviorInternalnumberLayer
        from plone.browserlayer import utils
        self.assertNotIn(ICollectiveBehaviorInternalnumberLayer, utils.registered_layers())



# -*- coding: utf-8 -*-

import unittest

from plone import api

from collective.behavior.internalnumber.testing import COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING  # noqa

from .. import TYPE_CONFIG
from ..browser.settings import get_settings, get_pt_settings, DxPortalTypesVocabulary


class TestSettings(unittest.TestCase):
    """Test settings."""

    layer = COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.tt1 = self.portal['tt1']
        api.portal.set_registry_record(TYPE_CONFIG, [{u'portal_type': u'glo_bal', u'uniqueness': False,
                                                      u'default_number': 1, u'default_expression': u'number'},
                                                     {u'portal_type': u'testtype', u'uniqueness': True,
                                                      u'default_number': 1, u'default_expression': u'number'}])

    def test_get_settings(self):
        self.assertDictEqual(get_settings(), {u'glo_bal': {u'u': False, u'nb': 1, 'expr': u'number'},
                                              u'testtype': {u'u': True, u'nb': 1, 'expr': u'number'}})

    def test_get_pt_settings(self):
        self.assertDictEqual(get_pt_settings('testtype'), {u'u': True, u'nb': 1, 'expr': u'number'})
        self.assertDictEqual(get_pt_settings('unknown'), {u'u': False, u'nb': 1, 'expr': u'number'})
        api.portal.set_registry_record(TYPE_CONFIG, [{u'portal_type': u'glo_bal', u'uniqueness': False,
                                                      u'default_number': 1, u'default_expression': u'number'}])
        self.assertDictEqual(get_pt_settings('testtype'), {u'u': False, u'nb': 1, 'expr': u'number'})

    def test_DxPortalTypesVocabulary(self):
        voc_inst = DxPortalTypesVocabulary()
        voc_list = [(t.value, t.title) for t in voc_inst(self.portal)]
        self.assertEqual(voc_list, [('glo_bal', u'Global configuration'), ('testtype', u'Test type')])
